@@ -23,14 +23,13 @@ function stripPrefix(string, prefix) {
     return string;
 }
 
-var Application = GObject.registerClass({
-    GTypeName: 'ExtensionPrefs_Application'
-}, class Application extends Gtk.Application {
+var Application = GObject.registerClass(
+class Application extends Gtk.Application {
     _init() {
         GLib.set_prgname('gnome-shell-extension-prefs');
         super._init({
             application_id: 'org.gnome.shell.ExtensionPrefs',
-            flags: Gio.ApplicationFlags.HANDLES_COMMAND_LINE
+            flags: Gio.ApplicationFlags.HANDLES_COMMAND_LINE,
         });
 
         this._startupUuid = null;
@@ -61,12 +60,12 @@ var Application = GObject.registerClass({
 
         let dialog = new Gtk.Window({
             modal: !this._skipMainWindow,
-            type_hint: Gdk.WindowTypeHint.DIALOG
+            type_hint: Gdk.WindowTypeHint.DIALOG,
         });
         dialog.set_titlebar(new Gtk.HeaderBar({
             show_close_button: true,
             title: row.name,
-            visible: true
+            visible: true,
         }));
 
         if (this._skipMainWindow) {
@@ -89,20 +88,20 @@ var Application = GObject.registerClass({
     _buildErrorUI(row, exc) {
         let scroll = new Gtk.ScrolledWindow({
             hscrollbar_policy: Gtk.PolicyType.NEVER,
-            propagate_natural_height: true
+            propagate_natural_height: true,
         });
 
         let box = new Gtk.Box({
             orientation: Gtk.Orientation.VERTICAL,
             spacing: 12,
             margin: 100,
-            margin_bottom: 60
+            margin_bottom: 60,
         });
         scroll.add(box);
 
         let label = new Gtk.Label({
             label: '<span size="x-large">%s</span>'.format(_("Something’s gone wrong")),
-            use_markup: true
+            use_markup: true,
         });
         label.get_style_context().add_class(Gtk.STYLE_CLASS_DIM_LABEL);
         box.add(label);
@@ -110,13 +109,13 @@ var Application = GObject.registerClass({
         label = new Gtk.Label({
             label: _("We’re very sorry, but there’s been a problem: the settings for this extension can’t be displayed. We recommend that you report the issue to the extension authors."),
             justify: Gtk.Justification.CENTER,
-            wrap: true
+            wrap: true,
         });
         box.add(label);
 
         let expander = new Expander({
             label: _("Technical Details"),
-            margin_top: 12
+            margin_top: 12,
         });
         box.add(expander);
 
@@ -127,14 +126,14 @@ var Application = GObject.registerClass({
 
         let buffer = new Gtk.TextBuffer({ text: errortext });
         let textview = new Gtk.TextView({
-            buffer: buffer,
+            buffer,
             wrap_mode: Gtk.WrapMode.WORD,
             monospace: true,
             editable: false,
             top_margin: 12,
             bottom_margin: 12,
             left_margin: 12,
-            right_margin: 12
+            right_margin: 12,
         });
 
         let toolbar = new Gtk.Toolbar();
@@ -150,7 +149,7 @@ var Application = GObject.registerClass({
 
         let copyButton = new Gtk.ToolButton({
             icon_name: 'edit-copy-symbolic',
-            tooltip_text: _("Copy Error")
+            tooltip_text: _("Copy Error"),
         });
         toolbar.add(copyButton);
 
@@ -159,15 +158,15 @@ var Application = GObject.registerClass({
             // markdown for pasting in gitlab issues
             let lines = [
                 `The settings of extension ${row.uuid} had an error:`,
-                '```',
+                '```', // '`' (xgettext throws up on odd number of backticks)
                 `${exc}`,
-                '```',
+                '```', // '`'
                 '',
                 'Stack trace:',
-                '```',
+                '```', // '`'
                 exc.stack.replace(/\n$/, ''), // stack without trailing newline
-                '```',
-                ''
+                '```', // '`'
+                '',
             ];
             clipboard.set_text(lines.join('\n'), -1);
         });
@@ -180,7 +179,7 @@ var Application = GObject.registerClass({
             label: _("Homepage"),
             tooltip_text: _("Visit extension homepage"),
             no_show_all: true,
-            visible: row.url != null
+            visible: row.url != null,
         });
         toolbar.add(urlButton);
 
@@ -190,7 +189,7 @@ var Application = GObject.registerClass({
         });
 
         let expandedBox = new Gtk.Box({
-            orientation: Gtk.Orientation.VERTICAL
+            orientation: Gtk.Orientation.VERTICAL,
         });
         expandedBox.add(textview);
         expandedBox.add(toolbar);
@@ -220,7 +219,7 @@ var Application = GObject.registerClass({
                             Gio.SettingsBindFlags.INVERT_BOOLEAN);
 
         this._mainStack = new Gtk.Stack({
-            transition_type: Gtk.StackTransitionType.CROSSFADE
+            transition_type: Gtk.StackTransitionType.CROSSFADE,
         });
         this._window.add(this._mainStack);
 
@@ -259,23 +258,15 @@ var Application = GObject.registerClass({
     }
 
     _onExtensionStateChanged(proxy, senderName, [uuid, newState]) {
+        let extension = ExtensionUtils.deserializeExtension(newState);
         let row = this._findExtensionRow(uuid);
+
         if (row) {
-            let { state } = ExtensionUtils.deserializeExtension(newState);
-            if (state == ExtensionState.UNINSTALLED)
+            if (extension.state === ExtensionState.UNINSTALLED)
                 row.destroy();
             return; // we only deal with new and deleted extensions here
         }
-
-        this._shellProxy.GetExtensionInfoRemote(uuid, ([serialized]) => {
-            let extension = ExtensionUtils.deserializeExtension(serialized);
-            if (!extension)
-                return;
-            // check the extension wasn't added in between
-            if (this._findExtensionRow(uuid) != null)
-                return;
-            this._addExtensionRow(extension);
-        });
+        this._addExtensionRow(extension);
     }
 
     _scanExtensions() {
@@ -361,20 +352,20 @@ var Expander = GObject.registerClass({
             'label', 'label', 'label',
             GObject.ParamFlags.READWRITE,
             null
-        )
-    }
+        ),
+    },
 }, class Expander extends Gtk.Box {
     _init(params = {}) {
         this._labelText = null;
 
         super._init(Object.assign(params, {
             orientation: Gtk.Orientation.VERTICAL,
-            spacing: 0
+            spacing: 0,
         }));
 
         this._frame = new Gtk.Frame({
             shadow_type: Gtk.ShadowType.IN,
-            hexpand: true
+            hexpand: true,
         });
 
         let eventBox = new Gtk.EventBox();
@@ -382,12 +373,12 @@ var Expander = GObject.registerClass({
 
         let hbox = new Gtk.Box({
             spacing: 6,
-            margin: 12
+            margin: 12,
         });
         eventBox.add(hbox);
 
         this._arrow = new Gtk.Image({
-            icon_name: 'pan-end-symbolic'
+            icon_name: 'pan-end-symbolic',
         });
         hbox.add(this._arrow);
 
@@ -397,7 +388,7 @@ var Expander = GObject.registerClass({
         this._revealer = new Gtk.Revealer();
 
         this._childBin = new Gtk.Frame({
-            shadow_type: Gtk.ShadowType.IN
+            shadow_type: Gtk.ShadowType.IN,
         });
         this._revealer.add(this._childBin);
 
@@ -415,7 +406,7 @@ var Expander = GObject.registerClass({
         this._gesture = new Gtk.GestureMultiPress({
             widget: this._frame,
             button: 0,
-            exclusive: true
+            exclusive: true,
         });
         this._gesture.connect('released', (gesture, nPress) => {
             if (nPress == 1)
@@ -460,7 +451,7 @@ class EmptyPlaceholder extends Gtk.Box {
         super._init({
             orientation: Gtk.Orientation.VERTICAL,
             spacing: 6,
-            margin: 32
+            margin: 32,
         });
 
         let image = new Gtk.Image({
@@ -468,15 +459,15 @@ class EmptyPlaceholder extends Gtk.Box {
             pixel_size: 96,
             visible: true,
             vexpand: true,
-            valign: Gtk.Align.END
+            valign: Gtk.Align.END,
         });
         image.get_style_context().add_class(Gtk.STYLE_CLASS_DIM_LABEL);
         this.add(image);
 
         let label = new Gtk.Label({
-            label: `<b><span size="x-large">${_("No Extensions Installed" )}</span></b>`,
+            label: `<b><span size="x-large">${_("No Extensions Installed")}</span></b>`,
             use_markup: true,
-            visible: true
+            visible: true,
         });
         label.get_style_context().add_class(Gtk.STYLE_CLASS_DIM_LABEL);
         this.add(label);
@@ -491,9 +482,9 @@ class EmptyPlaceholder extends Gtk.Box {
             visible: true,
             max_width_chars: 50,
             hexpand: true,
-            vexpand: (appInfo == null),
+            vexpand: appInfo == null,
             halign: Gtk.Align.CENTER,
-            valign: Gtk.Align.START
+            valign: Gtk.Align.START,
         });
         this.add(desc);
 
@@ -501,14 +492,14 @@ class EmptyPlaceholder extends Gtk.Box {
             let button = new Gtk.Button({
                 label: _("Browse in Software"),
                 image: new Gtk.Image({
-                    icon_name: "org.gnome.Software-symbolic"
+                    icon_name: "org.gnome.Software-symbolic",
                 }),
                 always_show_image: true,
                 margin_top: 12,
                 visible: true,
                 halign: Gtk.Align.CENTER,
                 valign: Gtk.Align.START,
-                vexpand: true
+                vexpand: true,
             });
             this.add(button);
 
@@ -527,13 +518,13 @@ class NoShellPlaceholder extends Gtk.Box {
             orientation: Gtk.Orientation.VERTICAL,
             spacing: 12,
             margin: 100,
-            margin_bottom: 60
+            margin_bottom: 60,
         });
 
         let label = new Gtk.Label({
             label: '<span size="x-large">%s</span>'.format(
                 _("Something’s gone wrong")),
-            use_markup: true
+            use_markup: true,
         });
         label.get_style_context().add_class(Gtk.STYLE_CLASS_DIM_LABEL);
         this.add(label);
@@ -541,7 +532,7 @@ class NoShellPlaceholder extends Gtk.Box {
         label = new Gtk.Label({
             label: _("We’re very sorry, but it was not possible to get the list of installed extensions. Make sure you are logged into GNOME and try again."),
             justify: Gtk.Justification.CENTER,
-            wrap: true
+            wrap: true,
         });
         this.add(label);
 
@@ -578,11 +569,11 @@ class ExtensionRow extends Gtk.ListBoxRow {
                     return;
 
                 this._extension = ExtensionUtils.deserializeExtension(newState);
-                let state = (this._extension.state == ExtensionState.ENABLED);
+                let state = this._extension.state == ExtensionState.ENABLED;
 
-                GObject.signal_handler_block(this._switch, this._notifyActiveId);
+                this._switch.block_signal_handler(this._notifyActiveId);
                 this._switch.state = state;
-                GObject.signal_handler_unblock(this._switch, this._notifyActiveId);
+                this._switch.unblock_signal_handler(this._notifyActiveId);
 
                 this._switch.sensitive = this._canToggle();
             });
@@ -649,7 +640,7 @@ class ExtensionRow extends Gtk.ListBoxRow {
         this._switch = new Gtk.Switch({
             valign: Gtk.Align.CENTER,
             sensitive: this._canToggle(),
-            state: this._extension.state === ExtensionState.ENABLED
+            state: this._extension.state === ExtensionState.ENABLED,
         });
         this._notifyActiveId = this._switch.connect('notify::active', () => {
             if (this._switch.active)
@@ -666,12 +657,12 @@ class ExtensionRow extends Gtk.ListBoxRow {
     }
 
     get prefsModule() {
+        // give extension prefs access to their own extension object
+        ExtensionUtils.getCurrentExtension = () => this._extension;
+
         if (!this._prefsModule) {
             ExtensionUtils.installImporter(this._extension);
 
-            // give extension prefs access to their own extension object
-            ExtensionUtils.getCurrentExtension = () => this._extension;
-
             this._prefsModule = this._extension.imports.prefs;
             this._prefsModule.init(this._extension.metadata);
         }
@@ -692,7 +683,7 @@ function initEnvironment() {
             log(`ERROR: ${s}`);
         },
 
-        userdatadir: GLib.build_filenamev([GLib.get_user_data_dir(), 'gnome-shell'])
+        userdatadir: GLib.build_filenamev([GLib.get_user_data_dir(), 'gnome-shell']),
     };
 
     String.prototype.format = Format.format;
