@@ -181,7 +181,7 @@ function loadRemoteSearchProviders(searchSettings, callback) {
             return -1;
 
         // finally, if both providers are found, return their order in the list
-        return (idxA - idxB);
+        return idxA - idxB;
     });
 
     callback(loadedProviders);
@@ -228,8 +228,7 @@ var RemoteSearchProvider = class {
         }
 
         if (gicon)
-            icon = new St.Icon({ gicon: gicon,
-                                 icon_size: size });
+            icon = new St.Icon({ gicon, icon_size: size });
         return icon;
     }
 
