@@ -25,7 +25,7 @@ var GeoclueAccuracyLevel = {
     CITY: 4,
     NEIGHBORHOOD: 5,
     STREET: 6,
-    EXACT: 8
+    EXACT: 8,
 };
 
 function accuracyLevelToString(accuracyLevel) {
@@ -42,9 +42,10 @@ const GeoclueManager = Gio.DBusProxy.makeProxyWrapper(GeoclueIface);
 
 var AgentIface = loadInterfaceXML('org.freedesktop.GeoClue2.Agent');
 
-var Indicator = class extends PanelMenu.SystemIndicator {
-    constructor() {
-        super();
+var Indicator = GObject.registerClass(
+class Indicator extends PanelMenu.SystemIndicator {
+    _init() {
+        super._init();
 
         this._settings = new Gio.Settings({ schema_id: LOCATION_SCHEMA });
         this._settings.connect(`changed::${ENABLED}`,
@@ -92,7 +93,7 @@ var Indicator = class extends PanelMenu.SystemIndicator {
                                            this._getMaxAccuracyLevel());
 
         authorizer.authorize(accuracyLevel => {
-            let ret = (accuracyLevel != GeoclueAccuracyLevel.NONE);
+            let ret = accuracyLevel != GeoclueAccuracyLevel.NONE;
             invocation.return_value(GLib.Variant.new('(bu)',
                                                      [ret, accuracyLevel]));
         });
@@ -222,7 +223,7 @@ var Indicator = class extends PanelMenu.SystemIndicator {
 
         this._permStoreProxy = proxy;
     }
-};
+});
 
 function clamp(value, min, max) {
     return Math.max(min, Math.min(max, value));
@@ -344,7 +345,7 @@ var AppAuthorizer = class {
 };
 
 var GeolocationDialog = GObject.registerClass({
-    Signals: { 'response': { param_types: [GObject.TYPE_UINT] } }
+    Signals: { 'response': { param_types: [GObject.TYPE_UINT] } },
 }, class GeolocationDialog extends ModalDialog.ModalDialog {
     _init(name, subtitle, reqAccuracyLevel) {
         super._init({ styleClass: 'geolocation-dialog' });
