@@ -48,19 +48,19 @@ static GParamSpec *props[N_PROPS] = { NULL, };
 
 struct _StIconPrivate
 {
-  ClutterActor *icon_texture;
-  ClutterActor *pending_texture;
-  guint         opacity_handler_id;
+  ClutterActor    *icon_texture;
+  ClutterActor    *pending_texture;
+  gulong           opacity_handler_id;
 
-  GIcon        *gicon;
-  gint          prop_icon_size;  /* icon size set as property */
-  gint          theme_icon_size; /* icon size from theme node */
-  gint          icon_size;       /* icon size we are using */
-  GIcon        *fallback_gicon;
+  GIcon           *gicon;
+  gint             prop_icon_size;  /* icon size set as property */
+  gint             theme_icon_size; /* icon size from theme node */
+  gint             icon_size;       /* icon size we are using */
+  GIcon           *fallback_gicon;
 
-  CoglPipeline *shadow_pipeline;
-  StShadow     *shadow_spec;
-  ClutterSize   shadow_size;
+  CoglPipeline    *shadow_pipeline;
+  StShadow        *shadow_spec;
+  graphene_size_t  shadow_size;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (StIcon, st_icon, ST_TYPE_WIDGET)
@@ -163,12 +163,13 @@ st_icon_dispose (GObject *gobject)
 }
 
 static void
-st_icon_paint (ClutterActor *actor)
+st_icon_paint (ClutterActor        *actor,
+               ClutterPaintContext *paint_context)
 {
   StIcon *icon = ST_ICON (actor);
   StIconPrivate *priv = icon->priv;
 
-  st_widget_paint_background (ST_WIDGET (actor));
+  st_widget_paint_background (ST_WIDGET (actor), paint_context);
 
   if (priv->icon_texture)
     {
@@ -177,16 +178,18 @@ st_icon_paint (ClutterActor *actor)
       if (priv->shadow_pipeline)
         {
           ClutterActorBox allocation;
+          CoglFramebuffer *framebuffer;
 
           clutter_actor_get_allocation_box (priv->icon_texture, &allocation);
+          framebuffer = clutter_paint_context_get_framebuffer (paint_context);
           _st_paint_shadow_with_opacity (priv->shadow_spec,
-                                         cogl_get_draw_framebuffer (),
+                                         framebuffer,
                                          priv->shadow_pipeline,
                                          &allocation,
                                          clutter_actor_get_paint_opacity (priv->icon_texture));
         }
 
-      clutter_actor_paint (priv->icon_texture);
+      clutter_actor_paint (priv->icon_texture, paint_context);
     }
 }
 
@@ -290,7 +293,7 @@ st_icon_clear_shadow_pipeline (StIcon *icon)
   StIconPrivate *priv = icon->priv;
 
   g_clear_pointer (&priv->shadow_pipeline, cogl_object_unref);
-  clutter_size_init (&priv->shadow_size, 0, 0);
+  graphene_size_init (&priv->shadow_size, 0, 0);
 }
 
 static void
@@ -317,7 +320,7 @@ st_icon_update_shadow_pipeline (StIcon *icon)
                                                    priv->icon_texture);
 
           if (priv->shadow_pipeline)
-            clutter_size_init (&priv->shadow_size, width, height);
+            graphene_size_init (&priv->shadow_size, width, height);
         }
     }
 }
@@ -369,8 +372,7 @@ opacity_changed_cb (GObject *object,
   StIcon *icon = user_data;
   StIconPrivate *priv = icon->priv;
 
-  g_signal_handler_disconnect (priv->pending_texture, priv->opacity_handler_id);
-  priv->opacity_handler_id = 0;
+  g_clear_signal_handler (&priv->opacity_handler_id, priv->pending_texture);
 
   st_icon_finish_update (icon);
 }
